/*******************************************************************************
* File Name: RadioController_INT.c
* Version 2.40
*
* Description:
*  This file provides all Interrupt Service Routine (ISR) for the SPI Master
*  component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "RadioController_PVT.h"

/* User code required at start of ISR */
/* `#START RadioController_ISR_START_DEF` */

/* `#END` */


/*******************************************************************************
* Function Name: RadioController_TX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for TX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  RadioController_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  RadioController_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified when exist data to
*  sending and FIFO Not Full.
*  RadioController_txBuffer[RadioController_TX_BUFFER_SIZE] - used to store
*  data to sending.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(RadioController_TX_ISR)
{
    #if(RadioController_TX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
    #endif /* (RadioController_TX_SOFTWARE_BUF_ENABLED) */

    /* User code required at start of ISR */
    /* `#START RadioController_TX_ISR_START` */

    /* `#END` */

    #if(RadioController_TX_SOFTWARE_BUF_ENABLED)
        /* Check if TX data buffer is not empty and there is space in TX FIFO */
        while(RadioController_txBufferRead != RadioController_txBufferWrite)
        {
            tmpStatus = RadioController_GET_STATUS_TX(RadioController_swStatusTx);
            RadioController_swStatusTx = tmpStatus;

            if(0u != (RadioController_swStatusTx & RadioController_STS_TX_FIFO_NOT_FULL))
            {
                if(0u == RadioController_txBufferFull)
                {
                   RadioController_txBufferRead++;

                    if(RadioController_txBufferRead >= RadioController_TX_BUFFER_SIZE)
                    {
                        RadioController_txBufferRead = 0u;
                    }
                }
                else
                {
                    RadioController_txBufferFull = 0u;
                }

                /* Move data from the Buffer to the FIFO */
                CY_SET_REG8(RadioController_TXDATA_PTR,
                    RadioController_txBuffer[RadioController_txBufferRead]);
            }
            else
            {
                break;
            }
        }

        if(RadioController_txBufferRead == RadioController_txBufferWrite)
        {
            /* TX Buffer is EMPTY: disable interrupt on TX NOT FULL */
            RadioController_TX_STATUS_MASK_REG &= ((uint8) ~RadioController_STS_TX_FIFO_NOT_FULL);
        }

    #endif /* (RadioController_TX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START RadioController_TX_ISR_END` */

    /* `#END` */
}


/*******************************************************************************
* Function Name: RadioController_RX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for RX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  RadioController_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer modified when FIFO contains
*  new data.
*  RadioController_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified when overflow occurred.
*  RadioController_rxBuffer[RadioController_RX_BUFFER_SIZE] - used to store
*  received data, modified when FIFO contains new data.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(RadioController_RX_ISR)
{
    #if(RadioController_RX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
        uint8 rxData;
    #endif /* (RadioController_RX_SOFTWARE_BUF_ENABLED) */

    /* User code required at start of ISR */
    /* `#START RadioController_RX_ISR_START` */

    /* `#END` */

    #if(RadioController_RX_SOFTWARE_BUF_ENABLED)

        tmpStatus = RadioController_GET_STATUS_RX(RadioController_swStatusRx);
        RadioController_swStatusRx = tmpStatus;

        /* Check if RX data FIFO has some data to be moved into the RX Buffer */
        while(0u != (RadioController_swStatusRx & RadioController_STS_RX_FIFO_NOT_EMPTY))
        {
            rxData = CY_GET_REG8(RadioController_RXDATA_PTR);

            /* Set next pointer. */
            RadioController_rxBufferWrite++;
            if(RadioController_rxBufferWrite >= RadioController_RX_BUFFER_SIZE)
            {
                RadioController_rxBufferWrite = 0u;
            }

            if(RadioController_rxBufferWrite == RadioController_rxBufferRead)
            {
                RadioController_rxBufferRead++;
                if(RadioController_rxBufferRead >= RadioController_RX_BUFFER_SIZE)
                {
                    RadioController_rxBufferRead = 0u;
                }

                RadioController_rxBufferFull = 1u;
            }

            /* Move data from the FIFO to the Buffer */
            RadioController_rxBuffer[RadioController_rxBufferWrite] = rxData;

            tmpStatus = RadioController_GET_STATUS_RX(RadioController_swStatusRx);
            RadioController_swStatusRx = tmpStatus;
        }

    #endif /* (RadioController_RX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START RadioController_RX_ISR_END` */

    /* `#END` */
}

/* [] END OF FILE */
