/*******************************************************************************
* File Name: RadioController.h
* Version 2.40
*
* Description:
*  Contains the function prototypes, constants and register definition
*  of the SPI Master Component.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_SPIM_RadioController_H)
#define CY_SPIM_RadioController_H

#include "cytypes.h"
#include "cyfitter.h"
#include "CyLib.h"

/* Check to see if required defines such as CY_PSOC5A are available */
/* They are defined starting with cy_boot v3.0 */
#if !defined (CY_PSOC5A)
    #error Component SPI_Master_v2_40 requires cy_boot v3.0 or later
#endif /* (CY_PSOC5A) */


/***************************************
*   Conditional Compilation Parameters
***************************************/

#define RadioController_INTERNAL_CLOCK             (1u)

#if(0u != RadioController_INTERNAL_CLOCK)
    #include "RadioController_IntClock.h"
#endif /* (0u != RadioController_INTERNAL_CLOCK) */

#define RadioController_MODE                       (1u)
#define RadioController_DATA_WIDTH                 (8u)
#define RadioController_MODE_USE_ZERO              (1u)
#define RadioController_BIDIRECTIONAL_MODE         (0u)

/* Internal interrupt handling */
#define RadioController_TX_BUFFER_SIZE             (4u)
#define RadioController_RX_BUFFER_SIZE             (4u)
#define RadioController_INTERNAL_TX_INT_ENABLED    (0u)
#define RadioController_INTERNAL_RX_INT_ENABLED    (0u)

#define RadioController_SINGLE_REG_SIZE            (8u)
#define RadioController_USE_SECOND_DATAPATH        (RadioController_DATA_WIDTH > RadioController_SINGLE_REG_SIZE)

#define RadioController_FIFO_SIZE                  (4u)
#define RadioController_TX_SOFTWARE_BUF_ENABLED    ((0u != RadioController_INTERNAL_TX_INT_ENABLED) && \
                                                     (RadioController_TX_BUFFER_SIZE > RadioController_FIFO_SIZE))

#define RadioController_RX_SOFTWARE_BUF_ENABLED    ((0u != RadioController_INTERNAL_RX_INT_ENABLED) && \
                                                     (RadioController_RX_BUFFER_SIZE > RadioController_FIFO_SIZE))


/***************************************
*        Data Struct Definition
***************************************/

/* Sleep Mode API Support */
typedef struct
{
    uint8 enableState;
    uint8 cntrPeriod;
    #if(CY_UDB_V0)
        uint8 saveSrTxIntMask;
        uint8 saveSrRxIntMask;
    #endif /* (CY_UDB_V0) */

} RadioController_BACKUP_STRUCT;


/***************************************
*        Function Prototypes
***************************************/

void  RadioController_Init(void)                           ;
void  RadioController_Enable(void)                         ;
void  RadioController_Start(void)                          ;
void  RadioController_Stop(void)                           ;

void  RadioController_EnableTxInt(void)                    ;
void  RadioController_EnableRxInt(void)                    ;
void  RadioController_DisableTxInt(void)                   ;
void  RadioController_DisableRxInt(void)                   ;

void  RadioController_Sleep(void)                          ;
void  RadioController_Wakeup(void)                         ;
void  RadioController_SaveConfig(void)                     ;
void  RadioController_RestoreConfig(void)                  ;

void  RadioController_SetTxInterruptMode(uint8 intSrc)     ;
void  RadioController_SetRxInterruptMode(uint8 intSrc)     ;
uint8 RadioController_ReadTxStatus(void)                   ;
uint8 RadioController_ReadRxStatus(void)                   ;
void  RadioController_WriteTxData(uint8 txData)  \
                                                            ;
uint8 RadioController_ReadRxData(void) \
                                                            ;
uint8 RadioController_GetRxBufferSize(void)                ;
uint8 RadioController_GetTxBufferSize(void)                ;
void  RadioController_ClearRxBuffer(void)                  ;
void  RadioController_ClearTxBuffer(void)                  ;
void  RadioController_ClearFIFO(void)                              ;
void  RadioController_PutArray(const uint8 buffer[], uint8 byteCount) \
                                                            ;

#if(0u != RadioController_BIDIRECTIONAL_MODE)
    void  RadioController_TxEnable(void)                   ;
    void  RadioController_TxDisable(void)                  ;
#endif /* (0u != RadioController_BIDIRECTIONAL_MODE) */

CY_ISR_PROTO(RadioController_TX_ISR);
CY_ISR_PROTO(RadioController_RX_ISR);


/**********************************
*   Variable with external linkage
**********************************/

extern uint8 RadioController_initVar;


/***************************************
*           API Constants
***************************************/

#define RadioController_TX_ISR_NUMBER     ((uint8) (RadioController_TxInternalInterrupt__INTC_NUMBER))
#define RadioController_RX_ISR_NUMBER     ((uint8) (RadioController_RxInternalInterrupt__INTC_NUMBER))

#define RadioController_TX_ISR_PRIORITY   ((uint8) (RadioController_TxInternalInterrupt__INTC_PRIOR_NUM))
#define RadioController_RX_ISR_PRIORITY   ((uint8) (RadioController_RxInternalInterrupt__INTC_PRIOR_NUM))


/***************************************
*    Initial Parameter Constants
***************************************/

#define RadioController_INT_ON_SPI_DONE    ((uint8) (0u   << RadioController_STS_SPI_DONE_SHIFT))
#define RadioController_INT_ON_TX_EMPTY    ((uint8) (0u   << RadioController_STS_TX_FIFO_EMPTY_SHIFT))
#define RadioController_INT_ON_TX_NOT_FULL ((uint8) (0u << \
                                                                           RadioController_STS_TX_FIFO_NOT_FULL_SHIFT))
#define RadioController_INT_ON_BYTE_COMP   ((uint8) (0u  << RadioController_STS_BYTE_COMPLETE_SHIFT))
#define RadioController_INT_ON_SPI_IDLE    ((uint8) (0u   << RadioController_STS_SPI_IDLE_SHIFT))

/* Disable TX_NOT_FULL if software buffer is used */
#define RadioController_INT_ON_TX_NOT_FULL_DEF ((RadioController_TX_SOFTWARE_BUF_ENABLED) ? \
                                                                        (0u) : (RadioController_INT_ON_TX_NOT_FULL))

/* TX interrupt mask */
#define RadioController_TX_INIT_INTERRUPTS_MASK    (RadioController_INT_ON_SPI_DONE  | \
                                                     RadioController_INT_ON_TX_EMPTY  | \
                                                     RadioController_INT_ON_TX_NOT_FULL_DEF | \
                                                     RadioController_INT_ON_BYTE_COMP | \
                                                     RadioController_INT_ON_SPI_IDLE)

#define RadioController_INT_ON_RX_FULL         ((uint8) (0u << \
                                                                          RadioController_STS_RX_FIFO_FULL_SHIFT))
#define RadioController_INT_ON_RX_NOT_EMPTY    ((uint8) (0u << \
                                                                          RadioController_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define RadioController_INT_ON_RX_OVER         ((uint8) (0u << \
                                                                          RadioController_STS_RX_FIFO_OVERRUN_SHIFT))

/* RX interrupt mask */
#define RadioController_RX_INIT_INTERRUPTS_MASK    (RadioController_INT_ON_RX_FULL      | \
                                                     RadioController_INT_ON_RX_NOT_EMPTY | \
                                                     RadioController_INT_ON_RX_OVER)
/* Nubmer of bits to receive/transmit */
#define RadioController_BITCTR_INIT            (((uint8) (RadioController_DATA_WIDTH << 1u)) - 1u)


/***************************************
*             Registers
***************************************/

#if(CY_PSOC3 || CY_PSOC5)
    #define RadioController_TXDATA_REG (* (reg8 *) \
                                                RadioController_BSPIM_sR8_Dp_u0__F0_REG)
    #define RadioController_TXDATA_PTR (  (reg8 *) \
                                                RadioController_BSPIM_sR8_Dp_u0__F0_REG)
    #define RadioController_RXDATA_REG (* (reg8 *) \
                                                RadioController_BSPIM_sR8_Dp_u0__F1_REG)
    #define RadioController_RXDATA_PTR (  (reg8 *) \
                                                RadioController_BSPIM_sR8_Dp_u0__F1_REG)
#else   /* PSOC4 */
    #if(RadioController_USE_SECOND_DATAPATH)
        #define RadioController_TXDATA_REG (* (reg16 *) \
                                          RadioController_BSPIM_sR8_Dp_u0__16BIT_F0_REG)
        #define RadioController_TXDATA_PTR (  (reg16 *) \
                                          RadioController_BSPIM_sR8_Dp_u0__16BIT_F0_REG)
        #define RadioController_RXDATA_REG (* (reg16 *) \
                                          RadioController_BSPIM_sR8_Dp_u0__16BIT_F1_REG)
        #define RadioController_RXDATA_PTR         (  (reg16 *) \
                                          RadioController_BSPIM_sR8_Dp_u0__16BIT_F1_REG)
    #else
        #define RadioController_TXDATA_REG (* (reg8 *) \
                                                RadioController_BSPIM_sR8_Dp_u0__F0_REG)
        #define RadioController_TXDATA_PTR (  (reg8 *) \
                                                RadioController_BSPIM_sR8_Dp_u0__F0_REG)
        #define RadioController_RXDATA_REG (* (reg8 *) \
                                                RadioController_BSPIM_sR8_Dp_u0__F1_REG)
        #define RadioController_RXDATA_PTR (  (reg8 *) \
                                                RadioController_BSPIM_sR8_Dp_u0__F1_REG)
    #endif /* (RadioController_USE_SECOND_DATAPATH) */
#endif     /* (CY_PSOC3 || CY_PSOC5) */

#define RadioController_AUX_CONTROL_DP0_REG (* (reg8 *) \
                                        RadioController_BSPIM_sR8_Dp_u0__DP_AUX_CTL_REG)
#define RadioController_AUX_CONTROL_DP0_PTR (  (reg8 *) \
                                        RadioController_BSPIM_sR8_Dp_u0__DP_AUX_CTL_REG)

#if(RadioController_USE_SECOND_DATAPATH)
    #define RadioController_AUX_CONTROL_DP1_REG  (* (reg8 *) \
                                        RadioController_BSPIM_sR8_Dp_u1__DP_AUX_CTL_REG)
    #define RadioController_AUX_CONTROL_DP1_PTR  (  (reg8 *) \
                                        RadioController_BSPIM_sR8_Dp_u1__DP_AUX_CTL_REG)
#endif /* (RadioController_USE_SECOND_DATAPATH) */

#define RadioController_COUNTER_PERIOD_REG     (* (reg8 *) RadioController_BSPIM_BitCounter__PERIOD_REG)
#define RadioController_COUNTER_PERIOD_PTR     (  (reg8 *) RadioController_BSPIM_BitCounter__PERIOD_REG)
#define RadioController_COUNTER_CONTROL_REG    (* (reg8 *) RadioController_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)
#define RadioController_COUNTER_CONTROL_PTR    (  (reg8 *) RadioController_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)

#define RadioController_TX_STATUS_REG          (* (reg8 *) RadioController_BSPIM_TxStsReg__STATUS_REG)
#define RadioController_TX_STATUS_PTR          (  (reg8 *) RadioController_BSPIM_TxStsReg__STATUS_REG)
#define RadioController_RX_STATUS_REG          (* (reg8 *) RadioController_BSPIM_RxStsReg__STATUS_REG)
#define RadioController_RX_STATUS_PTR          (  (reg8 *) RadioController_BSPIM_RxStsReg__STATUS_REG)

#define RadioController_CONTROL_REG            (* (reg8 *) \
                                      RadioController_BSPIM_BidirMode_SyncCtl_CtrlReg__CONTROL_REG)
#define RadioController_CONTROL_PTR            (  (reg8 *) \
                                      RadioController_BSPIM_BidirMode_SyncCtl_CtrlReg__CONTROL_REG)

#define RadioController_TX_STATUS_MASK_REG     (* (reg8 *) RadioController_BSPIM_TxStsReg__MASK_REG)
#define RadioController_TX_STATUS_MASK_PTR     (  (reg8 *) RadioController_BSPIM_TxStsReg__MASK_REG)
#define RadioController_RX_STATUS_MASK_REG     (* (reg8 *) RadioController_BSPIM_RxStsReg__MASK_REG)
#define RadioController_RX_STATUS_MASK_PTR     (  (reg8 *) RadioController_BSPIM_RxStsReg__MASK_REG)

#define RadioController_TX_STATUS_ACTL_REG     (* (reg8 *) RadioController_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define RadioController_TX_STATUS_ACTL_PTR     (  (reg8 *) RadioController_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define RadioController_RX_STATUS_ACTL_REG     (* (reg8 *) RadioController_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)
#define RadioController_RX_STATUS_ACTL_PTR     (  (reg8 *) RadioController_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)

#if(RadioController_USE_SECOND_DATAPATH)
    #define RadioController_AUX_CONTROLDP1     (RadioController_AUX_CONTROL_DP1_REG)
#endif /* (RadioController_USE_SECOND_DATAPATH) */


/***************************************
*       Register Constants
***************************************/

/* Status Register Definitions */
#define RadioController_STS_SPI_DONE_SHIFT             (0x00u)
#define RadioController_STS_TX_FIFO_EMPTY_SHIFT        (0x01u)
#define RadioController_STS_TX_FIFO_NOT_FULL_SHIFT     (0x02u)
#define RadioController_STS_BYTE_COMPLETE_SHIFT        (0x03u)
#define RadioController_STS_SPI_IDLE_SHIFT             (0x04u)
#define RadioController_STS_RX_FIFO_FULL_SHIFT         (0x04u)
#define RadioController_STS_RX_FIFO_NOT_EMPTY_SHIFT    (0x05u)
#define RadioController_STS_RX_FIFO_OVERRUN_SHIFT      (0x06u)

#define RadioController_STS_SPI_DONE           ((uint8) (0x01u << RadioController_STS_SPI_DONE_SHIFT))
#define RadioController_STS_TX_FIFO_EMPTY      ((uint8) (0x01u << RadioController_STS_TX_FIFO_EMPTY_SHIFT))
#define RadioController_STS_TX_FIFO_NOT_FULL   ((uint8) (0x01u << RadioController_STS_TX_FIFO_NOT_FULL_SHIFT))
#define RadioController_STS_BYTE_COMPLETE      ((uint8) (0x01u << RadioController_STS_BYTE_COMPLETE_SHIFT))
#define RadioController_STS_SPI_IDLE           ((uint8) (0x01u << RadioController_STS_SPI_IDLE_SHIFT))
#define RadioController_STS_RX_FIFO_FULL       ((uint8) (0x01u << RadioController_STS_RX_FIFO_FULL_SHIFT))
#define RadioController_STS_RX_FIFO_NOT_EMPTY  ((uint8) (0x01u << RadioController_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define RadioController_STS_RX_FIFO_OVERRUN    ((uint8) (0x01u << RadioController_STS_RX_FIFO_OVERRUN_SHIFT))

/* TX and RX masks for clear on read bits */
#define RadioController_TX_STS_CLR_ON_RD_BYTES_MASK    (0x09u)
#define RadioController_RX_STS_CLR_ON_RD_BYTES_MASK    (0x40u)

/* StatusI Register Interrupt Enable Control Bits */
/* As defined by the Register map for the AUX Control Register */
#define RadioController_INT_ENABLE     (0x10u) /* Enable interrupt from statusi */
#define RadioController_TX_FIFO_CLR    (0x01u) /* F0 - TX FIFO */
#define RadioController_RX_FIFO_CLR    (0x02u) /* F1 - RX FIFO */
#define RadioController_FIFO_CLR       (RadioController_TX_FIFO_CLR | RadioController_RX_FIFO_CLR)

/* Bit Counter (7-bit) Control Register Bit Definitions */
/* As defined by the Register map for the AUX Control Register */
#define RadioController_CNTR_ENABLE    (0x20u) /* Enable CNT7 */

/* Bi-Directional mode control bit */
#define RadioController_CTRL_TX_SIGNAL_EN  (0x01u)

/* Datapath Auxillary Control Register definitions */
#define RadioController_AUX_CTRL_FIFO0_CLR         (0x01u)
#define RadioController_AUX_CTRL_FIFO1_CLR         (0x02u)
#define RadioController_AUX_CTRL_FIFO0_LVL         (0x04u)
#define RadioController_AUX_CTRL_FIFO1_LVL         (0x08u)
#define RadioController_STATUS_ACTL_INT_EN_MASK    (0x10u)

/* Component disabled */
#define RadioController_DISABLED   (0u)


/***************************************
*       Macros
***************************************/

/* Returns true if componentn enabled */
#define RadioController_IS_ENABLED (0u != (RadioController_TX_STATUS_ACTL_REG & RadioController_INT_ENABLE))

/* Retuns TX status register */
#define RadioController_GET_STATUS_TX(swTxSts) ( (uint8)(RadioController_TX_STATUS_REG | \
                                                          ((swTxSts) & RadioController_TX_STS_CLR_ON_RD_BYTES_MASK)) )
/* Retuns RX status register */
#define RadioController_GET_STATUS_RX(swRxSts) ( (uint8)(RadioController_RX_STATUS_REG | \
                                                          ((swRxSts) & RadioController_RX_STS_CLR_ON_RD_BYTES_MASK)) )


/***************************************
*       Obsolete definitions
***************************************/

/* Following definitions are for version compatibility.
*  They are obsolete in SPIM v2_30.
*  Please do not use it in new projects
*/

#define RadioController_WriteByte   RadioController_WriteTxData
#define RadioController_ReadByte    RadioController_ReadRxData
void  RadioController_SetInterruptMode(uint8 intSrc)       ;
uint8 RadioController_ReadStatus(void)                     ;
void  RadioController_EnableInt(void)                      ;
void  RadioController_DisableInt(void)                     ;

/* Obsolete register names. Not to be used in new designs */
#define RadioController_TXDATA                 (RadioController_TXDATA_REG)
#define RadioController_RXDATA                 (RadioController_RXDATA_REG)
#define RadioController_AUX_CONTROLDP0         (RadioController_AUX_CONTROL_DP0_REG)
#define RadioController_TXBUFFERREAD           (RadioController_txBufferRead)
#define RadioController_TXBUFFERWRITE          (RadioController_txBufferWrite)
#define RadioController_RXBUFFERREAD           (RadioController_rxBufferRead)
#define RadioController_RXBUFFERWRITE          (RadioController_rxBufferWrite)

#define RadioController_COUNTER_PERIOD         (RadioController_COUNTER_PERIOD_REG)
#define RadioController_COUNTER_CONTROL        (RadioController_COUNTER_CONTROL_REG)
#define RadioController_STATUS                 (RadioController_TX_STATUS_REG)
#define RadioController_CONTROL                (RadioController_CONTROL_REG)
#define RadioController_STATUS_MASK            (RadioController_TX_STATUS_MASK_REG)
#define RadioController_STATUS_ACTL            (RadioController_TX_STATUS_ACTL_REG)

#define RadioController_INIT_INTERRUPTS_MASK  (RadioController_INT_ON_SPI_DONE     | \
                                                RadioController_INT_ON_TX_EMPTY     | \
                                                RadioController_INT_ON_TX_NOT_FULL_DEF  | \
                                                RadioController_INT_ON_RX_FULL      | \
                                                RadioController_INT_ON_RX_NOT_EMPTY | \
                                                RadioController_INT_ON_RX_OVER      | \
                                                RadioController_INT_ON_BYTE_COMP)
                                                
/* Following definitions are for version Compatibility.
*  They are obsolete in SPIM v2_40.
*  Please do not use it in new projects
*/

#define RadioController_DataWidth                  (RadioController_DATA_WIDTH)
#define RadioController_InternalClockUsed          (RadioController_INTERNAL_CLOCK)
#define RadioController_InternalTxInterruptEnabled (RadioController_INTERNAL_TX_INT_ENABLED)
#define RadioController_InternalRxInterruptEnabled (RadioController_INTERNAL_RX_INT_ENABLED)
#define RadioController_ModeUseZero                (RadioController_MODE_USE_ZERO)
#define RadioController_BidirectionalMode          (RadioController_BIDIRECTIONAL_MODE)
#define RadioController_Mode                       (RadioController_MODE)
#define RadioController_DATAWIDHT                  (RadioController_DATA_WIDTH)
#define RadioController_InternalInterruptEnabled   (0u)

#define RadioController_TXBUFFERSIZE   (RadioController_TX_BUFFER_SIZE)
#define RadioController_RXBUFFERSIZE   (RadioController_RX_BUFFER_SIZE)

#define RadioController_TXBUFFER       RadioController_txBuffer
#define RadioController_RXBUFFER       RadioController_rxBuffer

#endif /* (CY_SPIM_RadioController_H) */


/* [] END OF FILE */
